@@ -1,4 +1,4 @@
 library(testthat)
 library(GLMcat)
 
-test_check("GLMcat")
+# test_check("GLMcat")
